// K-3D
// Copyright (c) 1995-2004, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\author Timothy M. Shead (tshead@k-3d.com)
*/

#include <k3dsdk/axis.h>
#include <k3dsdk/imaterial.h>
#include <k3dsdk/object.h>
#include <k3dsdk/persistence.h>
#include <k3dsdk/measurement.h>
#include <k3dsdk/mesh_filter.h>
#include <k3dsdk/module.h>
#include <k3dsdk/mouse_event_observer.h>
#include <k3dsdk/noise.h>
#include <k3dsdk/plugins.h>
#include <k3dsdk/selection.h>
#include <k3dsdk/transform.h>

#include <iterator>

namespace libk3ddeformation
{

/////////////////////////////////////////////////////////////////////////////
// linear_point_noise_implementation

class linear_point_noise_implementation :
	public k3d::mesh_filter<k3d::persistent<k3d::object> >,
	public k3d::mouse_event_observer
{
	typedef k3d::mesh_filter<k3d::persistent<k3d::object> > base;

public:
	linear_point_noise_implementation(k3d::idocument& Document) :
		base(Document),
		k3d::mouse_event_observer("LMB drag to translate points"),
		m_use_x(k3d::init_name("use_x") + k3d::init_description("Use X [boolean]") + k3d::init_document(Document) + k3d::init_value(true)),
		m_use_y(k3d::init_name("use_y") + k3d::init_description("Use Y [boolean]") + k3d::init_document(Document) + k3d::init_value(true)),
		m_use_z(k3d::init_name("use_z") + k3d::init_description("Use Z [boolean]") + k3d::init_document(Document) + k3d::init_value(true)),
		m_move_x(k3d::init_name("move_x") + k3d::init_description("Move X [boolean]") + k3d::init_document(Document) + k3d::init_value(true)),
		m_move_y(k3d::init_name("move_y") + k3d::init_description("Move Y [boolean]") + k3d::init_document(Document) + k3d::init_value(true)),
		m_move_z(k3d::init_name("move_z") + k3d::init_description("Move Z [boolean]") + k3d::init_document(Document) + k3d::init_value(true)),
		m_frequency_x(k3d::init_name("frequency_x") + k3d::init_description("X Frequency [number]") + k3d::init_document(Document) + k3d::init_value(1.0) + k3d::init_precision(2) + k3d::init_step_increment(0.01) + k3d::init_units(typeid(k3d::measurement::scalar))),
		m_frequency_y(k3d::init_name("frequency_y") + k3d::init_description("Y Frequency [number]") + k3d::init_document(Document) + k3d::init_value(1.0) + k3d::init_precision(2) + k3d::init_step_increment(0.01) + k3d::init_units(typeid(k3d::measurement::scalar))),
		m_frequency_z(k3d::init_name("frequency_z") + k3d::init_description("Z Frequency [number]") + k3d::init_document(Document) + k3d::init_value(1.0) + k3d::init_precision(2) + k3d::init_step_increment(0.01) + k3d::init_units(typeid(k3d::measurement::scalar))),
		m_offset_x(k3d::init_name("offset_x") + k3d::init_description("X Offset [number]") + k3d::init_document(Document) + k3d::init_value(0.0) + k3d::init_precision(2) + k3d::init_step_increment(0.1) + k3d::init_units(typeid(k3d::measurement::distance))),
		m_offset_y(k3d::init_name("offset_y") + k3d::init_description("Y Offset [number]") + k3d::init_document(Document) + k3d::init_value(0.0) + k3d::init_precision(2) + k3d::init_step_increment(0.1) + k3d::init_units(typeid(k3d::measurement::distance))),
		m_offset_z(k3d::init_name("offset_z") + k3d::init_description("Z Offset [number]") + k3d::init_document(Document) + k3d::init_value(0.0) + k3d::init_precision(2) + k3d::init_step_increment(0.1) + k3d::init_units(typeid(k3d::measurement::distance))),
		m_amplitude_x(k3d::init_name("amplitude_x") + k3d::init_description("X Amplitude [number]") + k3d::init_document(Document) + k3d::init_value(1.0) + k3d::init_precision(2) + k3d::init_step_increment(0.1) + k3d::init_units(typeid(k3d::measurement::distance))),
		m_amplitude_y(k3d::init_name("amplitude_y") + k3d::init_description("Y Amplitude [number]") + k3d::init_document(Document) + k3d::init_value(1.0) + k3d::init_precision(2) + k3d::init_step_increment(0.1) + k3d::init_units(typeid(k3d::measurement::distance))),
		m_amplitude_z(k3d::init_name("amplitude_z") + k3d::init_description("Z Amplitude [number]") + k3d::init_document(Document) + k3d::init_value(1.0) + k3d::init_precision(2) + k3d::init_step_increment(0.1) + k3d::init_units(typeid(k3d::measurement::distance)))
	{
		enable_serialization(k3d::persistence::proxy(m_use_x));
		enable_serialization(k3d::persistence::proxy(m_use_y));
		enable_serialization(k3d::persistence::proxy(m_use_z));

		enable_serialization(k3d::persistence::proxy(m_move_x));
		enable_serialization(k3d::persistence::proxy(m_move_y));
		enable_serialization(k3d::persistence::proxy(m_move_z));

		enable_serialization(k3d::persistence::proxy(m_frequency_x));
		enable_serialization(k3d::persistence::proxy(m_frequency_y));
		enable_serialization(k3d::persistence::proxy(m_frequency_z));

		enable_serialization(k3d::persistence::proxy(m_offset_x));
		enable_serialization(k3d::persistence::proxy(m_offset_y));
		enable_serialization(k3d::persistence::proxy(m_offset_z));

		enable_serialization(k3d::persistence::proxy(m_amplitude_x));
		enable_serialization(k3d::persistence::proxy(m_amplitude_y));
		enable_serialization(k3d::persistence::proxy(m_amplitude_z));

		register_property(m_use_x);
		register_property(m_use_y);
		register_property(m_use_z);

		register_property(m_move_x);
		register_property(m_move_y);
		register_property(m_move_z);

		register_property(m_frequency_x);
		register_property(m_frequency_y);
		register_property(m_frequency_z);

		register_property(m_offset_x);
		register_property(m_offset_y);
		register_property(m_offset_z);

		register_property(m_amplitude_x);
		register_property(m_amplitude_y);
		register_property(m_amplitude_z);

		m_input_mesh.changed_signal().connect(SigC::slot(*this, &linear_point_noise_implementation::on_reset_geometry));

		m_use_x.changed_signal().connect(SigC::slot(*this, &linear_point_noise_implementation::on_reshape_geometry));
		m_use_y.changed_signal().connect(SigC::slot(*this, &linear_point_noise_implementation::on_reshape_geometry));
		m_use_z.changed_signal().connect(SigC::slot(*this, &linear_point_noise_implementation::on_reshape_geometry));

		m_move_x.changed_signal().connect(SigC::slot(*this, &linear_point_noise_implementation::on_reshape_geometry));
		m_move_y.changed_signal().connect(SigC::slot(*this, &linear_point_noise_implementation::on_reshape_geometry));
		m_move_z.changed_signal().connect(SigC::slot(*this, &linear_point_noise_implementation::on_reshape_geometry));

		m_frequency_x.changed_signal().connect(SigC::slot(*this, &linear_point_noise_implementation::on_reshape_geometry));
		m_frequency_y.changed_signal().connect(SigC::slot(*this, &linear_point_noise_implementation::on_reshape_geometry));
		m_frequency_z.changed_signal().connect(SigC::slot(*this, &linear_point_noise_implementation::on_reshape_geometry));

		m_offset_x.changed_signal().connect(SigC::slot(*this, &linear_point_noise_implementation::on_reshape_geometry));
		m_offset_y.changed_signal().connect(SigC::slot(*this, &linear_point_noise_implementation::on_reshape_geometry));
		m_offset_z.changed_signal().connect(SigC::slot(*this, &linear_point_noise_implementation::on_reshape_geometry));

		m_amplitude_x.changed_signal().connect(SigC::slot(*this, &linear_point_noise_implementation::on_reshape_geometry));
		m_amplitude_y.changed_signal().connect(SigC::slot(*this, &linear_point_noise_implementation::on_reshape_geometry));
		m_amplitude_z.changed_signal().connect(SigC::slot(*this, &linear_point_noise_implementation::on_reshape_geometry));

		m_output_mesh.need_data_signal().connect(SigC::slot(*this, &linear_point_noise_implementation::on_create_geometry));
	}

	bool OnLButtonDrag(const k3d::imouse_event_observer::event_state& State, const k3d::vector2& Current, const k3d::vector2& Last, const k3d::vector2& Start, const drag_type_t DragType)
	{
/*
		// Compute changes along each axis ...
		double dx = 0.0;
		double dy = 0.0;
		double dz = 0.0;

		// Some useful quantities ...
		const double mousex = (Current[0] - Last[0]);
		const double mousey = (Current[1] - Last[1]) * -1.0;
		const double mouse = fabs(mousex) > fabs(mousey) ? mousex : mousey;

		// If the user holds down either shift key, constrain translate factors to be equal ...
		if(State.modifiers.shift())
			{
				dx = dy = dz = mouse;
			}
		// If the user holds down either control key, only translate along the active axis ...
		else if(State.modifiers.control())
			{
				switch(State.active_axis)
					{
						case k3d::X:
							dx = mouse;
							break;
						case k3d::Y:
							dy = mouse;
							break;
						case k3d::Z:
							dz = mouse;
							break;
						default:
							assert_warning(0);
					}
			}
		// Otherwise, translate along the two non-active axes ...
		else
			{
				switch(State.active_axis)
					{
						case k3d::X:
							dz = mousex;
							dy = mousey;
							break;
						case k3d::Y:
							dx = mousex;
							dz = mousey;
							break;
						case k3d::Z:
							dx = mousex;
							dy = mousey;
							break;
						default:
							assert_warning(0);
					}
			}

		m_x.set_value(m_x.value() + dx);
		m_y.set_value(m_y.value() + dy);
		m_z.set_value(m_z.value() + dz);
*/
		return true;
	}

	void on_reset_geometry()
	{
		m_output_mesh.reset();
	}

	void on_reshape_geometry()
	{
		if(m_output_mesh.empty())
			return;

		const k3d::mesh* const input = m_input_mesh.property_value();
		if(!input)
			return;

		k3d::mesh* const output = const_cast<k3d::mesh*>(m_output_mesh.value());
		if(!output)
			return;

		reshape_geometry(*input, *output);

		m_output_mesh.changed_signal().emit();
	}

	k3d::mesh* on_create_geometry()
	{
		// If we don't have any input mesh, we're done ...
		const k3d::mesh* const input = m_input_mesh.property_value();
		if(!input)
			return 0;

		// Otherwise, we make a copy of the input mesh and modify the copy ...
		k3d::mesh* const output = new k3d::mesh();
		k3d::deep_copy(*input, *output);
		reshape_geometry(*input, *output);

		return output;
	}

	void reshape_geometry(const k3d::mesh& Source, k3d::mesh& Target)
	{
		// Sanity checks ...
		assert(Source.points.size() == Target.points.size());

		// Scale points ...
		const bool ignore_selection = !k3d::contains_selection(Source);

		const bool use_x = m_use_x.property_value();
		const bool use_y = m_use_y.property_value();
		const bool use_z = m_use_z.property_value();

		const bool move_x = m_move_x.property_value();
		const bool move_y = m_move_y.property_value();
		const bool move_z = m_move_z.property_value();

		const double frequency_x = m_frequency_x.property_value();
		const double frequency_y = m_frequency_y.property_value();
		const double frequency_z = m_frequency_z.property_value();

		const double offset_x = m_offset_x.property_value();
		const double offset_y = m_offset_y.property_value();
		const double offset_z = m_offset_z.property_value();

		const double amplitude_x = m_amplitude_x.property_value();
		const double amplitude_y = m_amplitude_y.property_value();
		const double amplitude_z = m_amplitude_z.property_value();

		for(unsigned long i = 0; i != Target.points.size(); ++i)
			{
				if(ignore_selection || Source.points[i]->selected)
					{
						const k3d::vector3 position = Source.points[i]->position;

						const k3d::vector3 noise = k3d::noise<k3d::vector3>(
							k3d::vector3(
								use_x ? offset_x + frequency_x * position[0] : 0.0,
								use_y ? offset_y + frequency_y * position[1] : 0.0,
								use_z ? offset_z + frequency_z * position[2] : 0.0));

						const k3d::vector3 offset =
							k3d::vector3(
								move_x ? amplitude_x * 2.0 * (noise[0] - 0.5) : 0.0,
								move_y ? amplitude_y * 2.0 * (noise[1] - 0.5) : 0.0,
								move_z ? amplitude_z * 2.0 * (noise[2] - 0.5) : 0.0);

						Target.points[i]->position = position + offset;
					}
				else
					{
						Target.points[i]->position = Source.points[i]->position;
					}
			}
	}

	k3d::iplugin_factory& factory()
	{
		return get_factory();
	}

	static k3d::iplugin_factory& get_factory()
	{
		static k3d::plugin_factory<
			k3d::document_plugin<linear_point_noise_implementation>,
			k3d::interface_list<k3d::imesh_source,
			k3d::interface_list<k3d::imesh_sink > > > factory(
				k3d::uuid(0xbbcaf2e7, 0xc45346bf, 0x9dfd92f2, 0xfb9e6d68),
				"LinearPointNoise",
				"Applies a linear noise offset to mesh points",
				"Objects",
				k3d::iplugin_factory::STABLE);

		return factory;
	}

private:
	k3d_data_property(bool, k3d::immutable_name, k3d::change_signal, k3d::with_undo, k3d::local_storage, k3d::no_constraint) m_use_x;
	k3d_data_property(bool, k3d::immutable_name, k3d::change_signal, k3d::with_undo, k3d::local_storage, k3d::no_constraint) m_use_y;
	k3d_data_property(bool, k3d::immutable_name, k3d::change_signal, k3d::with_undo, k3d::local_storage, k3d::no_constraint) m_use_z;

	k3d_data_property(bool, k3d::immutable_name, k3d::change_signal, k3d::with_undo, k3d::local_storage, k3d::no_constraint) m_move_x;
	k3d_data_property(bool, k3d::immutable_name, k3d::change_signal, k3d::with_undo, k3d::local_storage, k3d::no_constraint) m_move_y;
	k3d_data_property(bool, k3d::immutable_name, k3d::change_signal, k3d::with_undo, k3d::local_storage, k3d::no_constraint) m_move_z;

	k3d_measurement_property(double, k3d::immutable_name, k3d::change_signal, k3d::with_undo, k3d::local_storage, k3d::no_constraint) m_frequency_x;
	k3d_measurement_property(double, k3d::immutable_name, k3d::change_signal, k3d::with_undo, k3d::local_storage, k3d::no_constraint) m_frequency_y;
	k3d_measurement_property(double, k3d::immutable_name, k3d::change_signal, k3d::with_undo, k3d::local_storage, k3d::no_constraint) m_frequency_z;

	k3d_measurement_property(double, k3d::immutable_name, k3d::change_signal, k3d::with_undo, k3d::local_storage, k3d::no_constraint) m_offset_x;
	k3d_measurement_property(double, k3d::immutable_name, k3d::change_signal, k3d::with_undo, k3d::local_storage, k3d::no_constraint) m_offset_y;
	k3d_measurement_property(double, k3d::immutable_name, k3d::change_signal, k3d::with_undo, k3d::local_storage, k3d::no_constraint) m_offset_z;

	k3d_measurement_property(double, k3d::immutable_name, k3d::change_signal, k3d::with_undo, k3d::local_storage, k3d::no_constraint) m_amplitude_x;
	k3d_measurement_property(double, k3d::immutable_name, k3d::change_signal, k3d::with_undo, k3d::local_storage, k3d::no_constraint) m_amplitude_y;
	k3d_measurement_property(double, k3d::immutable_name, k3d::change_signal, k3d::with_undo, k3d::local_storage, k3d::no_constraint) m_amplitude_z;
};

/////////////////////////////////////////////////////////////////////////////
// linear_point_noise_factory

k3d::iplugin_factory& linear_point_noise_factory()
{
	return linear_point_noise_implementation::get_factory();
}

} // namespace libk3ddeformation


